;****************************************************************************
;* Program...: A_SEMA4.ASM
;* Author....: James R. Zack
;* Date......: 10-25-89
;* NOTICE....: COPYRIGHT (c) 1989, James R. Zack, ALL RIGHTS RESERVED
;* Purpose...: Provide a gallery of functions for creating and checking
;*             semaphore counts on Novell Netware Servers.
;*
;*
;****************************************************************************
PUBLIC S_OPEN, S_CLOSE, S_EXAMINE, S_SIGNAL, S_WAIT
EXTRN       __PARNI:FAR
EXTRN       __PARNL:FAR
EXTRN       __PARC:FAR
EXTRN       __PARCLEN:FAR
EXTRN       __RETNL:FAR
EXTRN       __RETNI:FAR
EXTRN       __RETL:FAR
EXTRN       __RET:FAR

DGROUP      GROUP    zackdata
zackdata    SEGMENT  'DATA'
sema_len    db       0
req_bufr    db       127 dup (?)
sema_count  db       0
zackdata    ENDS
zackprog    SEGMENT  'CODE'
            ASSUME   cs:zackprog,ds:DGROUP

;****************************************************************************
;* Function..: S_OPEN(<expC>,<expN>)
;* Author....: James R. Zack
;* Date......: 10-25-89
;* Purpose...: To open the specified semaphore, or create it if it does not exist.
;* Parameters: <expC> is the name of the semaphore to create or open. (1-127 characters)
;*             <expN> is the count to attach to the semaphore. (1-127 maximum)
;* Returns...: <expN> is the semaphore handle.
;* Note......: Be sure to keep track of the handle, as it is the only way you
;*             can examine or close the semaphore.
S_OPEN      PROC     FAR
            push     bp
            mov      bp,sp
            push     ds
            push     es
            push     si
            push     di
            mov      ax,1
            push     ax
            call     __PARCLEN
            add      sp,2
            mov      sema_len,al
            mov      ax,1
            push     ax
            call     __PARC
            add      sp,2
            cld
            mov      ds,dx
            mov      si,ax
            mov      ax,zackdata
            mov      es,ax
            mov      di,OFFSET req_bufr
            xor      cx,cx
            mov      cl,sema_len
            rep      movsb
;            mov      ax,DGROUP
;            mov      ds,ax
;            mov      es,ax
            mov      ax,2
            push     ax
            call     __PARNI
            add      sp,2
            mov      sema_count,al
            mov      ax,zackdata
            mov      ds,ax
            mov      ax,0c500h
            xor      cx,cx
            mov      dx,OFFSET req_bufr
            dec      dx
            mov      cl,sema_len
            inc      cx
            int      21h
            cmp      al,00
            jne      OPFAIL
            jmp      EXIT
OPFAIL:     xor      cx,cx
            xor      dx,dx
EXIT:       push     cx
            push     dx
            mov      ax,DGROUP
            mov      ds,ax
            call     __RETNL
            add      sp,4
            pop      di
            pop      si
            pop      es
            pop      ds
            pop      bp
            call     __RET
            ret
S_OPEN      ENDP

;****************************************************************************
;* Function..: S_CLOSE(<expN>)
;* Author....: James R. Zack
;* Date......: 10-25-89
;* Purpose...: To decrement the specified semaphore count, and delete the semaphore if count is zero.
;* Parameters: <expN> is the handle of the semaphore to close.
;* Returns...: <expL> is the success (True) or falure (False) of the operation.
S_CLOSE     PROC     FAR
            push     bp
            mov      bp,sp
            push     ds
            push     es
            push     si
            push     di
            mov      ax,1
            push     ax
            call     __PARNL
            add      sp,2
            mov      cx,dx
            mov      dx,ax
            mov      ax,0c504h
            int      21h
            cmp      al,00
            jne      OPFAIL4
            mov      ax,01h
            jmp      EXIT4
OPFAIL4:    mov      ax,00h
EXIT4:      push     ax
            call     __RETL
            add      sp,2
;            mov      ah,0d6h
;            mov      bx,0ffffh
;            int      21h
            pop      di
            pop      si
            pop      es
            pop      ds
            pop      bp
            call     __RET
            ret
S_CLOSE     ENDP

;****************************************************************************
;* Function..: S_EXAMINE(<expN>)
;* Author....: James R. Zack
;* Date......: 10-25-89
;* Purpose...: To determine the number of users currently accessing the semaphore.
;* Parameters: <expN> is the handle of the semaphore to examine.
;* Returns...: <expN> is the current open count.
;* Note......: The open count range is from +127 to -127.  If the number is a negative,
;*             then that is the number of users waiting for the semaphore.
S_EXAMINE   PROC     FAR
            push     bp
            mov      bp,sp
            push     ds
            push     es
            push     si
            push     di
            mov      ax,1
            push     ax
            call     __PARNL
            add      sp,2
            mov      cx,dx
            mov      dx,ax
            mov      ax,0c501h
            int      21h
            cmp      al,00
            jne      OPFAIL1
            jmp      EXIT1
OPFAIL1:    mov      dx,0000
EXIT1:      push     dx
            call     __RETNI
            add      sp,2
            pop      di
            pop      si
            pop      es
            pop      ds
            pop      bp
            call     __RET
            ret
S_EXAMINE   ENDP


;****************************************************************************
;* Function..: S_SIGNAL(<expN>)
;* Author....: James R. Zack
;* Date......: 10-25-89
;* Purpose...: To increment the value of the specified semaphore.
;* Parameters: <expN> is the handle of the semaphore to increment.
;* Returns...: <expL> is successful release of semaphore.

S_SIGNAL    PROC     FAR
            push     bp
            mov      bp,sp
            push     ds
            push     es
            push     si
            push     di
            mov      ax,1
            push     ax
            call     __PARNL
            add      sp,2
            mov      cx,dx
            mov      dx,ax
            mov      ax,0c503h
            int      21h
            cmp      al,00
            jne      OPFAIL2
            mov      ax,01h
            jmp      EXIT2
OPFAIL2:    mov      ax,00h
EXIT2:      push     ax
            call     __RETL
            add      sp,2
            pop      di
            pop      si
            pop      es
            pop      ds
            pop      bp
            call     __RET
            ret
S_SIGNAL    ENDP


;****************************************************************************
;* Function..: S_WAIT(<expN1>,<expN2>)
;* Author....: James R. Zack
;* Date......: 10-25-89
;* Purpose...: To wait in queue for next availible semaphore count.
;* Parameters: <expN> is the handle of the semaphore to increment.
;* Returns...: <expL> (True) is successful release of semaphore.
S_WAIT      PROC     FAR
            push     bp
            mov      bp,sp
            push     ds
            push     es
            push     si
            push     di
            mov      ax,1
            push     ax
            call     __PARNL
            add      sp,2
            mov      cx,dx
            mov      dx,ax
            mov      ax,2
            push     ax
            call     __PARNI
            add      sp,2
;            push     bp
            mov      bp,ax
            mov      ax,0c502h
            int      21h
;            pop      bp
            cmp      al,00
            jne      OPFAIL3
            mov      ax,01h
            jmp      EXIT3
OPFAIL3:    mov      ax,00h
EXIT3:      push     ax
            call     __RETL
            add      sp,2
            pop      di
            pop      si
            pop      es
            pop      ds
            pop      bp
            call     __RET
            ret
S_WAIT      ENDP

;****************************************************************************
zackprog    ENDS
            END

